import { FieldType } from './field-type';
import { FieldModels } from './field-models';
import { Link } from '../models/item/link.class';
import { RichTextResolver } from './rich-text-resolver.class';
export var Fields;
(function (Fields) {
    var TextField = /** @class */ (function () {
        /**
        * Represents text field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        */
        function TextField(name, value) {
            this.name = name;
            this.value = value;
            /**
            * Type of the field
            */
            this.type = FieldType.text;
            this.text = this.value;
        }
        ;
        return TextField;
    }());
    Fields.TextField = TextField;
    var MultipleChoiceField = /** @class */ (function () {
        /**
        * Represents multiple choice field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        */
        function MultipleChoiceField(name, value) {
            var _this = this;
            this.name = name;
            this.value = value;
            /**
            * Multiple choice options
            */
            this.options = [];
            /**
            * Type of the field
            */
            this.type = FieldType.multiple_choice;
            if (this.value && Array.isArray(this.value)) {
                this.value.forEach(function (option) {
                    var optionTemp = option;
                    _this.options.push(new FieldModels.MultipleChoiceOption(optionTemp.name, optionTemp.codename));
                });
            }
        }
        ;
        return MultipleChoiceField;
    }());
    Fields.MultipleChoiceField = MultipleChoiceField;
    var DateTimeField = /** @class */ (function () {
        /**
        * Represents date time field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        */
        function DateTimeField(name, value) {
            this.name = name;
            this.value = value;
            /**
            * Type of the field
            */
            this.type = FieldType.datetime;
            this.datetime = new Date(value);
        }
        ;
        return DateTimeField;
    }());
    Fields.DateTimeField = DateTimeField;
    var RichTextField = /** @class */ (function () {
        /**
        * Represents rich text field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        * @param {IContentItem[]} modularItems - Modular items
        * @param {ILink[]} links - Links in rich text field
        * @param {TypeResolverService} typeResolverService - Type resolver service
        * @param {boolean} enableAdvancedLogging - Indicates if advanced issues are logged in console
        * @param {IItemQueryConfig} itemQueryConfig - Item query config
        */
        function RichTextField(name, value, modularItems, links, typeResolverService, enableAdvancedLogging, itemQueryConfig) {
            this.name = name;
            this.value = value;
            this.modularItems = modularItems;
            this.links = links;
            this.typeResolverService = typeResolverService;
            this.enableAdvancedLogging = enableAdvancedLogging;
            this.itemQueryConfig = itemQueryConfig;
            /**
            * Type of the field
            */
            this.type = FieldType.rich_text;
            /**
            * List of modular content items used in this rich text field
            */
            this.items = [];
            this.items = modularItems;
        }
        ;
        RichTextField.prototype.getHtml = function () {
            // check if html was already resolved
            if (this.resolvedHtml) {
                return this.resolvedHtml;
            }
            var richTextHelper = new RichTextResolver(this.value, this.modularItems, this.links, this.typeResolverService, this.enableAdvancedLogging, this.itemQueryConfig);
            this.resolvedHtml = richTextHelper.resolveHtml();
            return this.resolvedHtml;
        };
        return RichTextField;
    }());
    Fields.RichTextField = RichTextField;
    var NumberField = /** @class */ (function () {
        /**
        * Represents number field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        */
        function NumberField(name, value) {
            this.name = name;
            this.value = value;
            /**
            * Type of the field
            */
            this.type = FieldType.number;
            this.number = value;
        }
        ;
        return NumberField;
    }());
    Fields.NumberField = NumberField;
    var AssetsField = /** @class */ (function () {
        /**
        * Represents asset field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        */
        function AssetsField(name, value) {
            var _this = this;
            this.name = name;
            this.value = value;
            /**
            * Type of the field
            */
            this.type = FieldType.asset;
            /**
            * List of assets used in this field
            */
            this.assets = [];
            if (!value) {
                throw Error("Cannot bind assets field because no value was provided");
            }
            if (!Array.isArray(value)) {
                throw Error("Cannot bind assets because the provided value is not an array");
            }
            this.value.forEach(function (asset) {
                var assetTemp = asset;
                _this.assets.push(new FieldModels.AssetModel(assetTemp.name, assetTemp.type, assetTemp.size, assetTemp.description, assetTemp.url));
            });
        }
        ;
        return AssetsField;
    }());
    Fields.AssetsField = AssetsField;
    var UrlSlugField = /** @class */ (function () {
        /**
        * Represents URL slug field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        * @param {IContentItem} item - Content item, required in order to get link object used by resolver
        * @param {((link: ILink) => string) | undefined} linkResolver - Callback used to resolve links
        * @param {boolean} enableAdvancedLogging - Indicates if advanced issues are logged in console
        */
        function UrlSlugField(name, value, item, linkResolver, enableAdvancedLogging) {
            this.name = name;
            this.value = value;
            this.item = item;
            this.linkResolver = linkResolver;
            this.enableAdvancedLogging = enableAdvancedLogging;
            /**
            * Type of the field
            */
            this.type = FieldType.url_slug;
        }
        ;
        UrlSlugField.prototype.getUrl = function () {
            if (!this.linkResolver) {
                if (this.enableAdvancedLogging) {
                    console.warn("You have to implement 'linkResolver' in your Model class or your query in order to get url of this item");
                }
                return null;
            }
            if (!this.item) {
                if (this.enableAdvancedLogging) {
                    console.warn("Cannot resolve link for type '" + this.type + "' because source item is not valid");
                }
                return null;
            }
            var url = this.linkResolver(new Link(this.item.system.id, this.item.system.codename, this.item.system.type, this.value));
            if (!url) {
                console.warn("'linkResolver' is configured, but url resolved for '" + this.type + "' type and '" + this.name + "' field resolved to an null/undefined value");
                return null;
            }
            return url;
        };
        return UrlSlugField;
    }());
    Fields.UrlSlugField = UrlSlugField;
    var TaxonomyField = /** @class */ (function () {
        /**
        * Represents number field of Kentico Cloud item
        * @constructor
        * @param {string} name - Name of the field
        * @param {string} value - Value of the field
        * @param {string | undefined} taxonomyGroup - Codename of the taxonomy group
        */
        function TaxonomyField(name, value, taxonomyGroup) {
            var _this = this;
            this.name = name;
            this.value = value;
            this.taxonomyGroup = taxonomyGroup;
            /**
            * Type of the field
            */
            this.type = FieldType.taxonomy;
            /**
            * List of assigned taxonomy terms
            */
            this.taxonomyTerms = [];
            if (!value) {
                throw Error("Cannot map taxonomy field because no value was provided");
            }
            if (!Array.isArray(value)) {
                throw Error("Cannot get taxonomy field because the provided value is not an array");
            }
            var taxonomyList = value;
            taxonomyList.forEach(function (term) {
                _this.taxonomyTerms.push(new FieldModels.TaxonomyTerm(term.name, term.codename));
            });
        }
        return TaxonomyField;
    }());
    Fields.TaxonomyField = TaxonomyField;
    ;
})(Fields || (Fields = {}));
//# sourceMappingURL=field-types.js.map