import * as parse5 from 'parse5/lib';
var RichTextResolver = /** @class */ (function () {
    /**
    * Rich text resolver
    * @constructor
    * @param {string} html - html to resolve
    * @param {IContentItem} modularItems - modular items
    * @param {ILink[]} links - links
    * @param {TypeResolverService} typeResolverService - Type resolver service used to access globally defined properties of models
    * @param {boolean} enableAdvancedLogging - Indicates if advanced issues are logged in console
    * @param {IItemQueryConfig} queryConfig - Query configuration
    */
    function RichTextResolver(html, modularItems, links, typeResolverService, enableAdvancedLogging, queryConfig) {
        this.html = html;
        this.modularItems = modularItems;
        this.links = links;
        this.typeResolverService = typeResolverService;
        this.enableAdvancedLogging = enableAdvancedLogging;
        this.queryConfig = queryConfig;
        /**
        * Type identifying nested modular content in Rich text fields
        */
        this.modularContentobjectType = 'application/kenticocloud';
        /**
         * This tag wil be used instead of 'object'
         */
        this.modularContentTagWrapper = 'div';
        /**
         * Attribute used to identify modular item based on its codename
         */
        this.modularContentCodenameAttributeName = 'data-codename';
        /**
         * Tag identifying links
         */
        this.linkTag = 'a';
        /**
         * Attributes that identifies if of the content item referenced in links
         */
        this.linkContentItemIdAttributeName = 'data-item-id';
    }
    ;
    /**
     * Resolves modular content inside the Rich text field.
     * Rich text resolved needs to be configured either on the model or query level
     */
    RichTextResolver.prototype.resolveHtml = function () {
        // resolve modular content nested within the rich text field
        // find the all 'object' tags
        // example: <object type="application/kenticocloud" data-type="item" data-codename="geralt"></object>
        var documentFragment = parse5.parseFragment(this.html);
        // get child nodes
        var childeNodes = this.getChildNodes(documentFragment);
        // recursively process all child nodes
        this.processChildNodes(childeNodes);
        // serialize document and get HTML
        var resolvedHtml = parse5.serialize(documentFragment);
        return resolvedHtml;
    };
    RichTextResolver.prototype.getChildNodes = function (documentFragment) {
        return documentFragment.childNodes;
    };
    RichTextResolver.prototype.processChildNodes = function (childNodes) {
        var _this = this;
        childNodes.forEach(function (node) {
            if (node.attrs) {
                var attributes = node.attrs;
                // process modular content
                _this.processModularContent(node, attributes);
                // process link
                _this.processLink(node, attributes);
            }
            if (node.childNodes) {
                // recursively process all nodes
                return _this.processChildNodes(_this.getChildNodes(node));
            }
        });
    };
    RichTextResolver.prototype.processLink = function (node, attributes) {
        var _this = this;
        if (node.nodeName !== this.linkTag) {
            // node is not a link
            return;
        }
        // get all links which have item it attribute, ignore all other links (they can be regular links in rich text)
        var contentItemIdAttribute = attributes.find(function (m) { return m.name === _this.linkContentItemIdAttributeName; });
        if (!contentItemIdAttribute) {
            // its either a regular link or the attribute is not defined
            return;
        }
        // get id of content item
        var contentItemId = contentItemIdAttribute.value;
        // find link with the id of content item
        var link = this.links.find(function (m) { return m.itemId === contentItemId; });
        if (!link) {
            if (this.enableAdvancedLogging) {
                console.warn("Cannot resolve URL for item '" + contentItemId + "' because no link with this id was found");
            }
            return;
        }
        // try to resolve link using the resolver passed through the query config
        var queryLinkResolver = this.queryConfig.linkResolver;
        var url;
        if (queryLinkResolver) {
            // try to resolve url using the query config
            url = queryLinkResolver(link);
        }
        if (!url) {
            // url was not resolved, try to find global resolver for this particular type
            // and apply its url resolver
            var emptyTypeItem = this.typeResolverService.createEmptyTypedObj(link.type);
            if (!emptyTypeItem) {
                throw Error("Cannot resolve link for '" + link.type + "' type because mapping failed (have you registered this type in your config?)");
            }
            var globalLinkResolver = emptyTypeItem.linkResolver;
            if (globalLinkResolver) {
                url = globalLinkResolver(link);
            }
        }
        // url still wasn't resolved
        if (!url) {
            if (this.enableAdvancedLogging) {
                console.warn("Url for content type '" + link.type + "' with id '" + link.itemId + "' resolved to null/undefiend");
            }
            return;
        }
        // assign url to 'href' attribute of the link
        var hrefAttribute = attributes.find(function (m) { return m.name === 'href'; });
        if (!hrefAttribute) {
            // href attribute is missing
            if (this.enableAdvancedLogging) {
                console.warn("Cannot set url '" + url + "' because 'href' attribute is not present in the <a> tag");
            }
            return;
        }
        hrefAttribute.value = url;
    };
    RichTextResolver.prototype.processModularContent = function (node, attributes) {
        var _this = this;
        var modularContentAttribute = attributes.find(function (m) { return m.value === _this.modularContentobjectType; });
        if (!modularContentAttribute) {
            // node is not of modular content type
            return;
        }
        // get codename of the modular content
        var modularItemCodenameAttribute = attributes.find(function (m) { return m.name === _this.modularContentCodenameAttributeName; });
        if (modularItemCodenameAttribute == null) {
            throw Error("The '" + this.modularContentCodenameAttributeName + "' attribute is missing and therefore modular content item cannot be retrieved");
        }
        var itemCodename = modularItemCodenameAttribute.value;
        // get modular content item
        var modularContentItem = this.modularItems.find(function (m) { return m.system.codename === itemCodename; });
        // check if modular content really exists
        if (!modularContentItem) {
            throw Error("Cannot resolve modular content in 'RichTextField' for '" + itemCodename + "' content item");
        }
        // replace 'object' tag name
        node.tagName = this.modularContentTagWrapper;
        // get html to replace object using Rich text resolver function
        var resolver = null;
        if (this.queryConfig.richTextResolver) {
            // use resolved defined by query if available
            resolver = this.queryConfig.richTextResolver;
        }
        else {
            // use default resolver defined in models
            if (modularContentItem.richTextResolver) {
                resolver = modularContentItem.richTextResolver;
            }
        }
        // check resolver
        if (resolver == null) {
            if (this.enableAdvancedLogging) {
                console.warn("Cannot resolve modular content of '" + modularContentItem.system.type + "' type in 'RichTextField' because no rich text resolved was configured");
            }
        }
        else {
            var replaceHtml = resolver(modularContentItem);
            var serializedHtml = parse5.parseFragment(replaceHtml);
            // add replaced html to node
            node.childNodes = serializedHtml.childNodes;
        }
    };
    return RichTextResolver;
}());
export { RichTextResolver };
//# sourceMappingURL=rich-text-resolver.class.js.map