import { CloudResponseDebug } from '../models/common/cloud-response-debug.class';
import { Pagination } from '../models/common/pagination.class';
import { ElementResponses } from '../models/element/responses';
import { ItemResponses } from '../models/item/responses';
import { TaxonomyResponses } from '../models/taxonomy/responses';
import { TypeResponses } from '../models/type/responses';
import { ElementMapService } from './element-map.service';
import { ItemMapService } from './item-map.service';
import { TaxonomyMapService } from './taxonomy-map.service';
import { TypeMapService } from './type-map.service';
var ResponseMapService = /** @class */ (function () {
    function ResponseMapService(config) {
        this.typeMapService = new TypeMapService();
        this.itemMapService = new ItemMapService(config);
        this.taxonomyMapService = new TaxonomyMapService();
        this.elementMapService = new ElementMapService();
    }
    /**
     * Gets response for getting a single type
     * @param response Response data
     */
    ResponseMapService.prototype.mapSingleTypeResponse = function (response) {
        var cloudResponse = response.data;
        // map type
        var type = this.typeMapService.mapSingleType(cloudResponse);
        return new TypeResponses.DeliveryTypeResponse(type, this.mapResponseDebug(response));
    };
    /**
     * Gets resposne for getting multiple types
     * @param response Response data
     * @param options Options
     */
    ResponseMapService.prototype.mapMultipleTypeResponse = function (response) {
        var cloudResponse = response.data;
        // map types
        var types = this.typeMapService.mapMultipleTypes(cloudResponse);
        // pagination
        var pagination = new Pagination(cloudResponse.pagination.skip, cloudResponse.pagination.limit, cloudResponse.pagination.count, cloudResponse.pagination.next_page);
        return new TypeResponses.DeliveryTypeListingResponse(types, pagination, this.mapResponseDebug(response));
    };
    /**
     * Gets response for getting single item
     * @param response Response data
     * @param queryConfig Query configuration
     */
    ResponseMapService.prototype.mapSingleResponse = function (response, queryConfig) {
        var cloudResponse = response.data;
        // map item
        var item = this.itemMapService.mapSingleItem(cloudResponse, queryConfig);
        return new ItemResponses.DeliveryItemResponse(item, this.mapResponseDebug(response));
    };
    /**
     * Gets response for getting multiple items
     * @param response Response data
     * @param queryConfig Query configuration
     */
    ResponseMapService.prototype.mapMultipleResponse = function (response, queryConfig) {
        var cloudResponse = response.data;
        // map items
        var items = this.itemMapService.mapMultipleItems(cloudResponse, queryConfig);
        // pagination
        var pagination = new Pagination(cloudResponse.pagination.skip, cloudResponse.pagination.limit, cloudResponse.pagination.count, cloudResponse.pagination.next_page);
        return new ItemResponses.DeliveryItemListingResponse(items, pagination, this.mapResponseDebug(response));
    };
    /**
     * Gets response for getting single taxonomy item
     * @param response Response data
     */
    ResponseMapService.prototype.mapTaxonomyResponse = function (response) {
        var cloudResponse = response.data;
        // map taxonomy
        var taxonomy = this.taxonomyMapService.mapTaxonomy(cloudResponse.system, cloudResponse.terms);
        return new TaxonomyResponses.TaxonomyResponse(taxonomy, this.mapResponseDebug(response));
    };
    /**
     * Gets response for getting multiples taxonomies
     * @param response Response data
     */
    ResponseMapService.prototype.mapTaxonomiesResponse = function (response) {
        var cloudResponse = response.data;
        // map taxonomies
        var taxonomies = this.taxonomyMapService.mapTaxonomies(cloudResponse.taxonomies);
        // pagination
        var pagination = new Pagination(cloudResponse.pagination.skip, cloudResponse.pagination.limit, cloudResponse.pagination.count, cloudResponse.pagination.next_page);
        return new TaxonomyResponses.TaxonomiesResponse(taxonomies, pagination, this.mapResponseDebug(response));
    };
    /**
    * Gets response for getting single content type element
    * @param response Response data
    */
    ResponseMapService.prototype.mapElementResponse = function (response) {
        var cloudResponse = response.data;
        // map element
        var element = this.elementMapService.mapElement(cloudResponse);
        return new ElementResponses.ElementResponse(element, this.mapResponseDebug(response));
    };
    ResponseMapService.prototype.mapResponseDebug = function (response) {
        if (!response) {
            throw Error("Cannot map 'debug' model from the response");
        }
        return new CloudResponseDebug(response.response);
    };
    return ResponseMapService;
}());
export { ResponseMapService };
//# sourceMappingURL=response-map.service.js.map