"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var field_decorators_1 = require("../fields/field-decorators");
var field_type_1 = require("../fields/field-type");
var field_types_1 = require("../fields/field-types");
var link_class_1 = require("../models/item/link.class");
var type_resolver_service_1 = require("./type-resolver.service");
var FieldMapService = /** @class */ (function () {
    function FieldMapService(config) {
        this.config = config;
        this.typeResolverService = new type_resolver_service_1.TypeResolverService(config);
    }
    /**
     * Maps all fields in given content item and returns strongly typed content item based on the resolver specified
     * in DeliveryClientConfig
     * @param item Item to map (raw response from Kentico Cloud)
     * @param modularContent Modular content sent along with item
     * @param queryConfig Query configuration
     */
    FieldMapService.prototype.mapFields = function (item, modularContent, queryConfig, processedItems) {
        var _this = this;
        if (!item) {
            throw Error("Cannot map fields because item is not defined");
        }
        if (!item.system) {
            throw Error("Cannot map item because it does not contain system attributes. This is an essential field and every item should have one.");
        }
        if (!item.elements) {
            throw Error("Cannot map elements of item with codename '" + item.system.codename + "'");
        }
        if (!processedItems) {
            throw Error("ProcessedItems need to be initialized");
        }
        if (!Array.isArray(processedItems)) {
            throw Error("ProcessedItems need to be an array of 'IContentItems'");
        }
        var properties = Object.getOwnPropertyNames(item.elements);
        var itemTyped;
        // check if resolver for this type is available
        if (this.typeResolverService.hasTypeResolver(item.system.type)) {
            itemTyped = this.typeResolverService.createTypedObj(item.system.type, item);
        }
        else {
            itemTyped = this.typeResolverService.createContentItem(item);
        }
        // add/taken item to processed items to avoid infinite recursion
        var processedItem = processedItems.find(function (m) { return m.system.codename === item.system.codename; });
        if (!processedItem) {
            processedItems.push(itemTyped);
        }
        properties.forEach(function (fieldName) {
            var field = item.elements[fieldName];
            var propertyName;
            // resolve field to a custom model property
            if (itemTyped.propertyResolver) {
                propertyName = itemTyped.propertyResolver(fieldName);
            }
            // if property hasn't been resolved, try with decorator
            if (!propertyName) {
                propertyName = field_decorators_1.FieldDecorators.getPropertyName(itemTyped, fieldName);
            }
            // if property name is null/empty, use elements codename
            if (!propertyName) {
                propertyName = fieldName;
            }
            itemTyped[propertyName] = _this.mapField(field, modularContent, itemTyped, queryConfig, processedItems);
        });
        return itemTyped;
    };
    FieldMapService.prototype.mapField = function (field, modularContent, item, queryConfig, processedItems) {
        if (field.type.toString() === field_type_1.FieldType.modular_content.toString()) {
            return this.mapModularField(field, modularContent, queryConfig, processedItems);
        }
        else if (field.type.toString() === field_type_1.FieldType.text.toString()) {
            return this.mapTextField(field);
        }
        else if (field.type.toString() === field_type_1.FieldType.asset.toString()) {
            return this.mapAssetsField(field);
        }
        else if (field.type.toString() === field_type_1.FieldType.number.toString()) {
            return this.mapNumberField(field);
        }
        else if (field.type.toString() === field_type_1.FieldType.multiple_choice.toString()) {
            return this.mapMultipleChoiceField(field);
        }
        else if (field.type.toString() === field_type_1.FieldType.datetime.toString()) {
            return this.mapDateTimeField(field);
        }
        else if (field.type.toString() === field_type_1.FieldType.rich_text.toString()) {
            return this.mapRichTextField(field, modularContent, queryConfig, processedItems);
        }
        else if (field.type.toString() === field_type_1.FieldType.url_slug.toString()) {
            return this.mapUrlSlugField(field, item, queryConfig);
        }
        else if (field.type.toString() === field_type_1.FieldType.taxonomy.toString()) {
            return this.mapTaxonomyField(field);
        }
        else {
            var err = "Unsupported field type '" + field.type + "'";
            if (this.config.enableAdvancedLogging) {
                console.warn(err, field);
            }
            throw Error(err);
        }
    };
    FieldMapService.prototype.mapRichTextField = function (field, modularContent, queryConfig, processedItems) {
        var _this = this;
        // get all modular content items nested in rich text
        var modularItems = [];
        if (field.modular_content) {
            if (Array.isArray(field.modular_content)) {
                field.modular_content.forEach(function (codename) {
                    // get modular item and check if it exists (it might not be included in response due to 'Depth' parameter)
                    var modularContentItem = modularContent[codename];
                    if (!modularContentItem) {
                        throw Error("Modular content item with codename '" + codename + "' is not present in Delivery response.\n                        This modular item was requested by '" + field.name + "' field.\n                        Error can usually be solved by increasing 'Depth' parameter of your query.");
                    }
                    var modularItem = _this.mapFields(modularContent[codename], modularContent, queryConfig, processedItems);
                    if (modularItem != null) {
                        modularItems.push(modularItem);
                    }
                });
            }
        }
        // extract and map links
        var links = this.mapRichTextLinks(field.links);
        return new field_types_1.Fields.RichTextField(field.name, field.value, modularItems, links, this.typeResolverService, this.config.enableAdvancedLogging, queryConfig);
    };
    FieldMapService.prototype.mapDateTimeField = function (field) {
        return new field_types_1.Fields.DateTimeField(field.name, field.value);
    };
    FieldMapService.prototype.mapMultipleChoiceField = function (field) {
        return new field_types_1.Fields.MultipleChoiceField(field.name, field.value);
    };
    FieldMapService.prototype.mapNumberField = function (field) {
        return new field_types_1.Fields.NumberField(field.name, field.value);
    };
    FieldMapService.prototype.mapTextField = function (field) {
        return new field_types_1.Fields.TextField(field.name, field.value);
    };
    FieldMapService.prototype.mapAssetsField = function (field) {
        return new field_types_1.Fields.AssetsField(field.name, field.value);
    };
    FieldMapService.prototype.mapTaxonomyField = function (field) {
        return new field_types_1.Fields.TaxonomyField(field.name, field.value, field.taxonomy_group);
    };
    FieldMapService.prototype.mapUrlSlugField = function (field, item, queryConfig) {
        var linkResolver = this.getLinkResolverForUrlSlugField(item, queryConfig);
        return new field_types_1.Fields.UrlSlugField(field.name, field.value, item, linkResolver, this.config.enableAdvancedLogging);
    };
    FieldMapService.prototype.mapModularField = function (field, modularContent, queryConfig, processedItems) {
        var _this = this;
        if (!field) {
            if (this.config.enableAdvancedLogging) {
                console.warn("Cannot map modular content field because field does not exist");
            }
            return null;
        }
        if (!field.value) {
            if (this.config.enableAdvancedLogging) {
                console.warn("Cannot map modular content of '" + field.name + "' because its value does not exist");
            }
            return null;
        }
        // modular content is always returned in an array
        var modularContentItems = [];
        var fieldModularContent = field.value;
        fieldModularContent.forEach(function (modularItemCodename) {
            var modularItem = modularContent[modularItemCodename];
            if (!modularItem) {
                if (_this.config.enableAdvancedLogging) {
                    console.warn("Cannot map '" + field.name + "' modular content item. Make sure you use 'DepthParameter' in case your modular content is nested.");
                }
            }
            // try to map only if the modular item was present in response
            if (modularItem) {
                // add/taken item to processed items to avoid infinite recursion
                var processedItem = processedItems.find(function (m) { return m.system.codename === modularItem.system.codename; });
                if (processedItem) {
                    modularContentItems.push(processedItem);
                }
                else {
                    var newModularItem = _this.mapFields(modularItem, modularContent, queryConfig, processedItems);
                    modularContentItems.push(newModularItem);
                    processedItem = newModularItem;
                }
            }
        });
        return modularContentItems;
    };
    FieldMapService.prototype.getLinkResolverForUrlSlugField = function (item, queryConfig) {
        // link resolver defined by the query config (= by calling method) has priority over type's global link resolver
        var linkResolver = undefined;
        if (queryConfig.linkResolver) {
            linkResolver = queryConfig.linkResolver;
        }
        else {
            linkResolver = item.linkResolver;
        }
        return linkResolver;
    };
    FieldMapService.prototype.mapRichTextLinks = function (linksJson) {
        var links = [];
        for (var _i = 0, _a = Object.keys(linksJson); _i < _a.length; _i++) {
            var linkItemId = _a[_i];
            var linkRaw = linksJson[linkItemId];
            links.push(new link_class_1.Link(linkItemId, linkRaw.codename, linkRaw.type, linkRaw.url_slug));
        }
        return links;
    };
    return FieldMapService;
}());
exports.FieldMapService = FieldMapService;
//# sourceMappingURL=field-map.service.js.map