"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var AjaxObservable_1 = require("rxjs/observable/dom/AjaxObservable");
var throw_1 = require("rxjs/observable/throw");
var cloud_error_class_1 = require("../models/common/cloud-error.class");
var header_class_1 = require("../models/common/header.class");
var retry_strategy_1 = require("./helpers/retry-strategy");
var response_map_service_1 = require("./response-map.service");
var QueryService = /** @class */ (function () {
    function QueryService(
    /**
     * Delivery client configuration
     */
    config, 
    /**
     * Http service for fetching data
     */
    httpService, 
    /**
     * Information about the SDK
     * This can contain information from both this & Node SDK for internal logging with 'SDKID' header
     */
    sdkInfo) {
        this.config = config;
        this.httpService = httpService;
        this.sdkInfo = sdkInfo;
        /**
         * Default number of retry attempts when user did not set any
         */
        this.defaultRetryAttempts = 0;
        /**
         * Excluded status code from retry functionality
         */
        this.retryExcludedStatuses = [404, 401];
        /**
         * Header name for SDK usage
         */
        this.sdkVersionHeader = 'X-KC-SDKID';
        /**
        * Default base Url to Kentico Delivery API
        */
        this.defaultBaseDeliveryApiUrl = 'https://deliver.kenticocloud.com';
        /**
        * Default preview url to Kentico Delivery API
        */
        this.defaultPreviewDeliveryApiUrl = 'https://preview-deliver.kenticocloud.com';
        /**
         * Name of the header used when 'wait for loading new content' feature is used
         */
        this.waitForLoadingNewContentHeader = 'X-KC-Wait-For-Loading-New-Content';
        if (!config) {
            throw Error("Invalid configuration has been provided");
        }
        this.responseMapService = new response_map_service_1.ResponseMapService(config);
    }
    /**
     * Gets url based on the action, query configuration and options (parameters)
     * @param action Action (= url part) that will be hit
     * @param queryConfig Query configuration
     * @param options Query options
     */
    QueryService.prototype.getUrl = function (action, queryConfig, options) {
        return this.addOptionsToUrl(this.getBaseUrl(queryConfig) + action, options);
    };
    /**
     * Gets single item from given url
     * @param url Url used to get single item
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getSingleItem = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapSingleResponse(response, queryConfig);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
    * Gets multiple items from given url
    * @param url Url used to get multiple items
    * @param queryConfig Query configuration
    */
    QueryService.prototype.getMultipleItems = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapMultipleResponse(response, queryConfig);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
     * Gets single content type from given url
     * @param url Url used to get single type
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getSingleType = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapSingleTypeResponse(response);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
     * Gets multiple content types from given url
     * @param url Url used to get multiple types
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getMultipleTypes = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapMultipleTypeResponse(response);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
     * Gets single taxonomy from given url
     * @param url Url used to get single taxonomy
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getTaxonomy = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapTaxonomyResponse(response);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
    * Gets multiple taxonomies from given url
    * @param url Url used to get multiple taxonomies
    * @param queryConfig Query configuration
    */
    QueryService.prototype.getTaxonomies = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapTaxonomiesResponse(response);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
    * Gets single content type element from given url
    * @param url Url used to get single content type element
    * @param queryConfig Query configuration
    */
    QueryService.prototype.getElement = function (url, queryConfig) {
        var _this = this;
        return this.getResponse(url, queryConfig)
            .map(function (response) {
            return _this.responseMapService.mapElementResponse(response);
        })
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
     * Gets proper set of headers for given request.
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getHeaders = function (queryConfig) {
        var headers = [];
        // add SDK Id header for monitoring SDK usage
        headers.push(this.getSdkIdHeader());
        if (this.isPreviewModeEnabled(queryConfig) && this.isSecuredModeEnabled(queryConfig)) {
            throw Error("Preview & secured mode cannot be used at the same time.");
        }
        // add preview header is required
        if (this.isPreviewModeEnabled(queryConfig)) {
            headers.push(this.getAuthorizationHeader(this.config.previewApiKey));
        }
        // add secured mode header is required
        if (this.isSecuredModeEnabled(queryConfig)) {
            headers.push(this.getAuthorizationHeader(this.config.securedApiKey));
        }
        // add 'X-KC-Wait-For-Loading-New-Content' header if required
        if (queryConfig.waitForLoadingNewContent) {
            headers.push(new header_class_1.Header(this.waitForLoadingNewContentHeader, 'true'));
        }
        return headers;
    };
    /**
     * Handles given error
     * @param error Error to be handled
     */
    QueryService.prototype.handleError = function (error) {
        if (this.config.enableAdvancedLogging) {
            console.error(error);
        }
        if (error instanceof AjaxObservable_1.AjaxError) {
            var xhrResponse = error.xhr.response;
            if (!xhrResponse) {
                return error;
            }
            // return Cloud specific error
            return new cloud_error_class_1.CloudError(xhrResponse.message, xhrResponse.request_id, xhrResponse.error_code, xhrResponse.specific_code, error);
        }
        return error;
    };
    /**
     * Indicates if current query should use preview mode
     * @param queryConfig Query configuration
     */
    QueryService.prototype.isPreviewModeEnabled = function (queryConfig) {
        if (queryConfig.usePreviewMode != null) {
            return queryConfig.usePreviewMode;
        }
        return this.config.enablePreviewMode === true;
    };
    /**
    * Indicates if current query should use secured mode
    * @param queryConfig Query configuration
    */
    QueryService.prototype.isSecuredModeEnabled = function (queryConfig) {
        if (queryConfig.useSecuredMode != null) {
            return queryConfig.useSecuredMode;
        }
        return this.config.enableSecuredMode === true;
    };
    /**
     * Gets preview or standard URL based on client and query configuration
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getDeliveryUrl = function (queryConfig) {
        if (this.isPreviewModeEnabled(queryConfig)) {
            if (!this.config.previewApiKey) {
                throw Error("You have to configure 'previewApiKey' to use 'preview' mode");
            }
            // use custom base / preview url if its configured
            if (this.config.basePreviewUrl) {
                return this.config.basePreviewUrl;
            }
            // use default preview url
            return this.defaultPreviewDeliveryApiUrl;
        }
        // use custom base / preview url if its configured
        if (this.config.baseUrl) {
            return this.config.baseUrl;
        }
        return this.defaultBaseDeliveryApiUrl;
    };
    /**
     * Gets base URL of the request including the project Id
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getBaseUrl = function (queryConfig) {
        return this.getDeliveryUrl(queryConfig) + '/' + this.config.projectId;
    };
    /**
     * Adds query parameters to given url
     * @param url Url to which options will be added
     * @param options Query parameters to add
     */
    QueryService.prototype.addOptionsToUrl = function (url, options) {
        if (options) {
            options.forEach(function (filter) {
                if (url.indexOf('?') > -1) {
                    url = url + '&' + filter.getParam() + '=' + filter.getParamValue();
                }
                else {
                    url = url + '?' + filter.getParam() + '=' + filter.getParamValue();
                }
            });
        }
        return url;
    };
    /**
     * Http get response
     * @param url Url of request
     * @param queryConfig Query configuration
     */
    QueryService.prototype.getResponse = function (url, queryConfig) {
        var _this = this;
        // hold the attempt count
        var attempt = 1;
        return this.httpService.get(url, this.getHeaders(queryConfig))
            .map(function (response) { return response; })
            .retryWhen(retry_strategy_1.retryStrategy.strategy({
            maxRetryAttempts: this.config.retryAttempts ? this.config.retryAttempts : this.defaultRetryAttempts,
            excludedStatusCodes: this.retryExcludedStatuses
        }))
            .catch(function (err) {
            return throw_1._throw(_this.handleError(err));
        });
    };
    /**
     * Gets authorization header. This is used for 'preview' functionality
     */
    QueryService.prototype.getAuthorizationHeader = function (key) {
        if (!key) {
            throw Error("Cannot get authorization header because key is undefined");
        }
        // authorization header required for preview mode
        return new header_class_1.Header('authorization', "bearer " + key);
    };
    /**
     * Header identifying SDK type & version for internal purposes of Kentico
     */
    QueryService.prototype.getSdkIdHeader = function () {
        return new header_class_1.Header(this.sdkVersionHeader, this.sdkInfo.host + ";" + this.sdkInfo.name + ";" + this.sdkInfo.version);
    };
    return QueryService;
}());
exports.QueryService = QueryService;
//# sourceMappingURL=query.service.js.map